"""
 mod_mail_test.py : Module, test whether nameservers are alive

 Copyright (c) 2009, Karel Slany    (slany AT fit.vutbr.cz)
                     Zdenek Vasicek (vasicek AT fit.vutbr.cz)
                     Marek Vavrusa  (xvavru00 AT stud.fit.vutbr.cz)
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of the organization nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
"""

from interface import TestModule

import mod_mx_recs

import socket


def has_smtp(ipstr):
    # open socket
    af = socket.AF_INET
    if ipstr.find(":") != -1:
        af = socket.AF_INET6
    sock = socket.socket(af, socket.SOCK_STREAM)
    sock.settimeout(1)
    # read
    try:
        sock.connect((ipstr, 25))
    except:
        return False, ""
    # attempt to read
    char = ""
    data = ""
    has_smtp = False
    try:
        while True:
            char = sock.recv(1)
            data += char
            if char == '\n':
                has_smtp = True
                break
    except:
        pass
    sock.close()
    return has_smtp, data.strip()


class Module(TestModule):
    """Check mail servers"""

    def __init__(self, args, modenv):
        TestModule.__init__(self,args,modenv,"Test mail servers")
        self.mx_env = self.get_module_modenv(mod_mx_recs)

    def execute(self):

        res= {}

        # check option
        if ((not self.args.options.mxrecs)
            or (not self.args.options.mailcheck)):
            self.set_my_modenv(res)
            return True

        res["mailservs"] = {}

        for servname in self.mx_env["dnservs"].keys():

            # for ipv4 addresses
            if (not self.args.options.noipv4
                and ("ipv4" in self.mx_env["dnservs"][servname].keys())):
                for addr in self.mx_env["dnservs"][servname]["ipv4"].keys():

                    if ("MX" in self.mx_env["dnservs"][servname]["ipv4"][addr].keys()):
                        for mx in self.mx_env["dnservs"][servname]["ipv4"][addr]["MX"]:
                            name = mx["server-name"]
                            if name not in res["mailservs"].keys():
                                res["mailservs"][name] = {}
                                res["mailservs"][name]["ipv4"] = []
                                if (mx["ipv4"]):
                                    self.verb(2, "adding mailserver", name, mx["ipv4"])
                                    res["mailservs"][name]["ipv4"].append(mx["ipv4"])
                                res["mailservs"][name]["ipv6"] = []
                                if (mx["ipv6"]):
                                    self.verb(2, "adding mailserver", name, mx["ipv6"])
                                    res["mailservs"][name]["ipv6"].append(mx["ipv6"])
                            else:
                                if (mx["ipv4"]
                                    and (mx["ipv4"] not in res["mailservs"][name]["ipv4"])):
                                    self.verb(2, "adding mailserver", name, mx["ipv4"])
                                    res["mailservs"][name]["ipv4"].append(mx["ipv4"])
                                if (mx["ipv6"]
                                    and (mx["ipv6"] not in res["mailservs"][name]["ipv6"])):
                                    self.verb(2, "adding mailserver", name, mx["ipv6"])
                                    res["mailservs"][name]["ipv6"].append(mx["ipv6"])

            # for ipv6 adresses
            if (self.args.options.resolver6
                and ("ipv6" in self.mx_env["dnservs"][servname].keys())):
                for addr in self.mx_env["dnservs"][servname]["ipv6"].keys():

                    if ("MX" in self.mx_env["dnservs"][servname]["ipv6"][addr].keys()):
                        for mx in self.mx_env["dnservs"][servname]["ipv6"][addr]["MX"]:
                            name = mx["server-name"]
                            if name not in res["mailservs"].keys():
                                res["mailservs"][name] = {}
                                res["mailservs"][name]["ipv4"] = []
                                if (mx["ipv4"]):
                                    self.verb(2, "adding mailserver", name, mx["ipv4"])
                                    res["mailservs"][name]["ipv4"].append(mx["ipv4"])
                                res["mailservs"][name]["ipv6"] = []
                                if (mx["ipv6"]):
                                    self.verb(2, "adding mailserver", name, mx["ipv6"])
                                    res["mailservs"][name]["ipv6"].append(mx["ipv6"])
                            else:
                                if (mx["ipv4"]
                                    and (mx["ipv4"] not in res["mailservs"][name]["ipv4"])):
                                    self.verb(2, "adding mailserver", name, mx["ipv4"])
                                    res["mailservs"][name]["ipv4"].append(mx["ipv4"])
                                if (mx["ipv6"]
                                    and (mx["ipv6"] not in res["mailservs"][name]["ipv6"])):
                                    self.verb(2, "adding mailserver", name, mx["ipv6"])
                                    res["mailservs"][name]["ipv6"].append(mx["ipv6"])

        # querry by a system default resolver
        resolver = self.ldns.ldns_resolver.new_frm_file(None)

        for name in res["mailservs"].keys():
            # try to resolve additional IPs

            addr = resolver.get_addr_by_name(name, self.ldns.LDNS_RR_CLASS_IN, self.ldns.LDNS_RD)

            for addr_rr in addr.rrs():

                if (not self.args.options.noipv4):
                    if (addr_rr.get_type() == self.ldns.LDNS_RR_TYPE_A):
                        if (str(addr_rr.rdf(0)) not in res["mailservs"][name]["ipv4"]):
                            res["mailservs"][name]["ipv4"].append(str(addr_rr.rdf(0)))
                            self.verb(2, "adding resolved mailserver", name, str(addr_rr.rdf(0)))
                else:
                    res["mailservs"][name]["ipv4"] = []

                if (self.args.options.ipv6):
                    if (addr_rr.get_type() == self.ldns.LDNS_RR_TYPE_AAAA):
                        if (str(addr_rr.rdf(0)) not in res["mailservs"][name]["ipv6"]):
                            res["mailservs"][name]["ipv6"].append(str(addr_rr.rdf(0)))
                            self.verb(2, "adding resolved mailserver", name, str(addr_rr.rdf(0)))
                else:
                    res["mailservs"][name]["ipv6"] = []
        self.verb(2, "")

        # compute symmary
        all_valid = True
        for name, addrlst in res["mailservs"].iteritems():
            if (not self.args.options.noipv4):
                for addr in addrlst["ipv4"]:
            	    smtp,data = has_smtp(addr)
                    self.verb(1, "Mail server (%s %s) has valid SMTP: " % (name, addr), smtp)
                    self.verb(2, "Mail server (%s %s) response: " % (name, addr), data)
                    all_valid = all_valid and smtp
            if (self.args.options.ipv6):
                for addr in addrlst["ipv6"]:
            	    smtp,data = has_smtp(addr)
                    self.verb(1, "Mail server (%s %s) has valid SMTP: " % (name, addr), smtp)
                    self.verb(2, "Mail server (%s %s) response: " % (name, addr), data)
                    all_valid = all_valid and smtp
        self.verb(1, "")

        # print summary
        if (all_valid):
            self.verb(0, "All found mail servers have valid SMTP: yes")
        else:
            self.verb(0, "All found mail servers have valid SMTP: no")
        self.verb(0, "")

        self.set_my_modenv(res)
        return True
